﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;


namespace MVRPlugin {
	public class LightTogglerNoEmissives : MVRScript {


        protected UIDynamicSlider lightIntensitySlider;
        protected UIDynamicToggle diffOffsetSlider;

        protected List<Light> origLights = new List<Light>();
        protected List<Material> emittingMaterials = new List<Material>();
        protected List<Light> customLights = new List<Light>();
        protected JSONStorableFloat jLightIntensity;
        protected JSONStorableBool jLightEnabled;
		protected List<Renderer> renderers = new List<Renderer>();

		public override void Init() {
			try {
				BuildUIControls();
				StartCoroutine(WaitForAssetBundle());

			} catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		protected void BuildUIControls() {
            jLightEnabled = new JSONStorableBool("lightEnabled", true, SetLightIntensity);
            jLightEnabled.storeType = JSONStorableParam.StoreType.Full;
            RegisterBool(jLightEnabled);
            CreateToggle(jLightEnabled, true);
        }

		protected bool IsValidMaterial(Material mat) {
			return true;
		}

		protected IEnumerator WaitForAssetBundle() {
			List<Light> allLights = new List<Light>();
            List<Renderer> allRenderers = new List<Renderer>();

            float waitForSeconds = 3.0f;
			float waited = 0f;
			while (allLights.Count < 1 && waited <= waitForSeconds) {
				containingAtom.GetComponentsInChildren<Light>(false, allLights);

                waited += 0.2f;
				yield return new WaitForSeconds(0.2f);
			}
            waited = 0f;
            while (allRenderers.Count < 1 && waited <= waitForSeconds)
            {
                containingAtom.GetComponentsInChildren<Renderer>(false, allRenderers);
                waited += 0.2f;
                yield return new WaitForSeconds(0.2f);
            }
			GetLights(allLights);
            GetMaterials(allRenderers);
            SetLightIntensity(jLightEnabled);

        }

        protected void GetMaterials(List<Renderer> renderers)
        {
            foreach(Renderer renderer in renderers)
            {
                Material[] mats = renderer.materials;
                for (int i = 0; i < mats.Length; i++)
                {
                    if (mats[i].IsKeywordEnabled("_EMISSION"))
                    {
                        emittingMaterials.Add(mats[i]);
                    }
                }
            }

        }

        protected void GetLights(List<Light> lights) {
            if (lights.Count == 0)
            {
                return;
            }
            foreach (Light light in lights)
            {
                customLights.Add(light);
            }
        }


		void OnDestroy() {
		}

        void turnLightsOff()
        {
            foreach (Light light in customLights)
            {
                light.enabled = false;
            }
            foreach(Material mat in emittingMaterials)
            {
             //   mat.DisableKeyword("_EMISSION");
            }
        }

        void turnLightsOn()
        {
            foreach (Light light in customLights)
            {
                light.enabled = true;
            }

            foreach (Material mat in emittingMaterials)
            {
               // mat.EnableKeyword("_EMISSION");
            }
        }

		protected void SetLightIntensity(JSONStorableBool jl) {
            {
                if (jl.val == true)
                {
                    turnLightsOn();
                } else
                {
                    turnLightsOff();
                }
            }
		}

	}
}