using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace MVRPlugin {
	public class ExtraltodeusPumpPlugin : MVRScript {

		private Vector3 sp;
		private Vector3 startPos;
		private Vector3 oldPosition;

		private Atom target;
		private float step;
		private JSONStorableFloat lengthSlider;
		private JSONStorableFloat strengthSlider;
		private JSONStorableFloat speedSlider;
		private JSONStorableFloat pumpValueJSON;



		private float oldResult;



		protected void SyncAtomChoices() {
			List<string> atomChoices = new List<string>();
			atomChoices.Add("None");
			foreach (string atomUID in SuperController.singleton.GetAtomUIDs()) {
				atomChoices.Add(atomUID);
			}
			atomJSON.choices = atomChoices;
		}

		// receiver Atom
		protected Atom receivingAtom;
		protected void SyncAtom(string atomUID) {
			List<string> receiverChoices = new List<string>();
			receiverChoices.Add("None");
			if (atomUID != null) {
				receivingAtom = SuperController.singleton.GetAtomByUid(atomUID);
				if (receivingAtom != null) {
					foreach (string receiverChoice in receivingAtom.GetStorableIDs()) {
						receiverChoices.Add(receiverChoice);
						//SuperController.LogMessage("Found receiver " + receiverChoice);
					}
				}
			} else {
				receivingAtom = null;
			}
			receiverJSON.choices = receiverChoices;
			receiverJSON.val = "None";
		}
		protected JSONStorableStringChooser atomJSON;

		protected string _missingReceiverStoreId = "";
		protected void CheckMissingReceiver() {
			if (_missingReceiverStoreId != "" && receivingAtom != null) {
				JSONStorable missingReceiver = receivingAtom.GetStorableByID(_missingReceiverStoreId);
				if (missingReceiver != null) {
					//Debug.Log("Found late-loading receiver " + _missingReceiverStoreId);
					string saveTargetName = _receiverTargetName;
					SyncReceiver(_missingReceiverStoreId);
					_missingReceiverStoreId = "";
					insideRestore = true;
					receiverTargetJSON.val = saveTargetName;
					insideRestore = false;
				}
			}
		}



				// receiver JSONStorable
				protected JSONStorable receiver;
				protected void SyncReceiver(string receiverID) {
					List<string> receiverTargetChoices = new List<string>();
					receiverTargetChoices.Add("None");
					if (receivingAtom != null && receiverID != null) {
						receiver = receivingAtom.GetStorableByID(receiverID);
						if (receiver != null) {
							foreach (string floatParam in receiver.GetFloatParamNames()) {
								receiverTargetChoices.Add(floatParam);
							}
						} else if (receiverID != "None") {
							// some storables can be late loaded, like skin, clothing, hair, etc so must keep track of missing receiver
							//Debug.Log("Missing receiver " + receiverID);
							_missingReceiverStoreId = receiverID;
						}
					} else {
						receiver = null;
					}
					receiverTargetJSON.choices = receiverTargetChoices;
					receiverTargetJSON.val = "None";
				}
				protected JSONStorableStringChooser receiverJSON;

				// receiver target parameter
				protected string _receiverTargetName;
				protected JSONStorableFloat receiverTarget;
				protected void SyncReceiverTarget(string receiverTargetName) {
					_receiverTargetName = receiverTargetName;
					receiverTarget = null;
					if (receiver != null && receiverTargetName != null) {
						receiverTarget = receiver.GetFloatJSONParam(receiverTargetName);
						if (receiverTarget != null) {
							lowerValueJSON.min = receiverTarget.min;
							lowerValueJSON.max = receiverTarget.max;
							upperValueJSON.min = receiverTarget.min;
							upperValueJSON.max = receiverTarget.max;
							currentValueJSON.min = receiverTarget.min;
							currentValueJSON.max = receiverTarget.max;
							targetValueJSON.min = receiverTarget.min;
							targetValueJSON.max = receiverTarget.max;
							if (!insideRestore) {
								// only sync up val if not in restore
								lowerValueJSON.val = receiverTarget.val;
								upperValueJSON.val = receiverTarget.val;
								currentValueJSON.val = receiverTarget.val;
								targetValueJSON.val = receiverTarget.val;
							}
						}
					}
				}
				protected JSONStorableStringChooser receiverTargetJSON;

				protected JSONStorableFloat downSpeedJSON;
				protected JSONStorableFloat upSpeedJSON;
				protected JSONStorableFloat durationJSON;

				protected JSONStorableFloat lowerValueJSON;
				protected JSONStorableFloat upperValueJSON;
				protected JSONStorableFloat targetValueJSON;
				protected JSONStorableFloat currentValueJSON;
				protected UIDynamicButton speedController;


				protected JSONStorableBool pumpOnce;






		protected JSONStorableFloat smoothLoopFactor;
		private Dictionary<int, float> smoothLoops = new Dictionary<int, float>();


		private float toElapse = 0;

		public override void Init() {
			try
			{
				oldResult = 0;
				oldPosition = containingAtom.transform.position;


				speedController = CreateButton("Pump", false);
				speedController.button.onClick.AddListener(delegate ()
				{
						toElapse = durationJSON.val;
				});

				pumpOnce = new JSONStorableBool("Pump once", false, (bool on)=>
				{
					pumpOnce.val = false;
					toElapse = durationJSON.val;
				});
				RegisterBool(pumpOnce);



				atomJSON = new JSONStorableStringChooser("atom", SuperController.singleton.GetAtomUIDs(), null, "Atom", SyncAtom);
				RegisterStringChooser(atomJSON);
				SyncAtomChoices();
				UIDynamicPopup dp = CreateScrollablePopup(atomJSON,true);
				dp.popupPanelHeight = 1100f;
				// want to always resync the atom choices on opening popup since atoms can be added/removed
				dp.popup.onOpenPopupHandlers += SyncAtomChoices;

				// make receiver selector
				receiverJSON = new JSONStorableStringChooser("receiver", null, null, "Receiver", SyncReceiver);
				RegisterStringChooser(receiverJSON);
				dp = CreateScrollablePopup(receiverJSON,true);
				dp.popupPanelHeight = 960f;

				// make receiver target selector
				receiverTargetJSON = new JSONStorableStringChooser("receiverTarget", null, null, "Target", SyncReceiverTarget);
				RegisterStringChooser(receiverTargetJSON);
				dp = CreateScrollablePopup(receiverTargetJSON,true);
				dp.popupPanelHeight = 820f;

				// set atom to current atom to initialize
				atomJSON.val = containingAtom.uid;

				CreateSpacer().height = 10f;


				smoothLoopFactor = new JSONStorableFloat("Smooth factor", 50f, 1f, 100f, true);
				RegisterFloat(smoothLoopFactor);
				CreateSlider(smoothLoopFactor, true);

				// morphSlider.slider.onValueChanged.AddListener(delegate (float mixValue)


				downSpeedJSON = new JSONStorableFloat("Down speed", 0.5f, 0f, 10f, true);
				RegisterFloat(downSpeedJSON);
				CreateSlider(downSpeedJSON, true);

				upSpeedJSON = new JSONStorableFloat("Up speed", 1f, 0f, 10f, true);
				RegisterFloat(upSpeedJSON);
				CreateSlider(upSpeedJSON, true);

				durationJSON = new JSONStorableFloat("Duration", 20f, 1f, 100f, true);
				RegisterFloat(durationJSON);
				CreateSlider(durationJSON, true);

				// lower val
				lowerValueJSON = new JSONStorableFloat("lowerValue", 0f, 0f, 1f, false);
				RegisterFloat(lowerValueJSON);
				CreateSlider(lowerValueJSON, true);

				// upper val
				upperValueJSON = new JSONStorableFloat("upperValue", 1f, 0.1f, 1f, false);
				RegisterFloat(upperValueJSON);
				CreateSlider(upperValueJSON, true);

				CreateSpacer().height = 10f;

				pumpValueJSON = new JSONStorableFloat("Pump value", 0f, 0f, 1f, false, false);
				// don't register - this is for viewing only and is generated
				UIDynamicSlider dsi = CreateSlider(pumpValueJSON, false);
				dsi.defaultButtonEnabled = false;
				dsi.quickButtonsEnabled = false;

				// target val
				targetValueJSON = new JSONStorableFloat("targetValue", 0f, 0f, 1f, false, false);
				// don't register - this is for viewing only and is generated
				// UIDynamicSlider ds = CreateSlider(targetValueJSON, true);
				// ds.defaultButtonEnabled = false;
				// ds.quickButtonsEnabled = false;
				// //
				// current val
				currentValueJSON = new JSONStorableFloat("currentValue", 0f, 0f, 1f, false, false);
				// don't register - this is for viewing only and is generated
				// ds = CreateSlider(currentValueJSON, true);
				// ds.defaultButtonEnabled = false;
				// ds.quickButtonsEnabled = false;

			}

			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}


		private int currentLoop = 0;
		private float pump = 0;
		private float speed = 0;

		protected void FixedUpdate() {

				if(toElapse > 0)
				{
					speed = upSpeedJSON.val;
					toElapse--;
				}
				else
				{
					speed = -downSpeedJSON.val;
				}

				if (toElapse<=0)
				{
					toElapse = 0;
				}

				pump = pump+speed/100f;

				if (pump>1f)
				{
					pump = 1f;
				}

				if (pump<0)
				{
					pump = 0;
				}

				pumpValueJSON.val = pump;

				if (receiverTarget != null) {

				int smoothFactor = (int)Math.Ceiling(smoothLoopFactor.val);


					float result = pump * (upperValueJSON.val - lowerValueJSON.val) + lowerValueJSON.val ;

					if (result > upperValueJSON.val)
						result = upperValueJSON.val;

					if (result < lowerValueJSON.val)
							result = lowerValueJSON.val;


					oldResult = result;

					smoothLoops[currentLoop] = result;

					int divider = 0;
					float total = 0;
					for (int i = 0; i < smoothFactor; i++)
					{
						if (smoothLoops.ContainsKey(i))
						{
							divider++;
							total = total+smoothLoops[i];
						}
						else
						{
							smoothLoops[i] = 0;
							divider++;
						}
					}

					result = total/divider;
					currentValueJSON.val = result;

					if (currentLoop < smoothFactor)
					{
						currentLoop++;
					}
					else
					{
						currentLoop = 0;
					}
				}

				// check for receivers that might have been missing on load due to asynchronous load of some assets like skin, clothing, hair
				CheckMissingReceiver();
				if (receiverTarget != null) {
					receiverTarget.val  = currentValueJSON.val;
				}




		}

		void Update()
		{
			try
			{


			}
			catch (Exception e)
			{
					SuperController.LogError("Exception caught: " + e);
			}
		}

		void OnDestroy() {
		}

	}
}
